package test.drs_tst.recs.plugins.scheduling.old;

import gridsim.dcworms.DCWormsTags;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import schedframe.events.scheduling.SchedulingEvent;
import schedframe.resources.ResourceStatus;
import schedframe.resources.computing.Node;
import schedframe.resources.computing.ComputingResource;
import schedframe.resources.computing.Core;
import schedframe.resources.computing.Processor;
import schedframe.resources.computing.profiles.energy.power.StandardPowerStateName;
import schedframe.resources.units.ProcessingElements;
import schedframe.resources.units.ResourceUnit;
import schedframe.resources.units.ResourceUnitName;
import schedframe.resources.units.StandardResourceUnitName;
import schedframe.scheduling.manager.resources.ClusterResourceManager;
import schedframe.scheduling.manager.resources.ResourceManager;
import schedframe.scheduling.manager.tasks.JobRegistry;
import schedframe.scheduling.plan.SchedulingPlanInterface;
import schedframe.scheduling.plan.impl.SchedulingPlan;
import schedframe.scheduling.plugin.ModuleList;
import schedframe.scheduling.queue.TaskQueue;
import schedframe.scheduling.queue.TaskQueueList;
import schedframe.scheduling.tasks.TaskInterface;
import test.article.recs.plugins.scheduling.RecsSP;

public class RecsExclusivenessNodePowManSP extends RecsSP {


	public SchedulingPlanInterface<?> schedule(SchedulingEvent event, TaskQueueList queues, JobRegistry jobRegistry,
			ResourceManager resManager, ModuleList modules) {

		ClusterResourceManager resourceManager = (ClusterResourceManager) resManager;
		SchedulingPlan plan = new SchedulingPlan();

		switch (event.getType()) {
		case START_TASK_EXECUTION:
		case TASK_FINISHED:

			TaskQueue q = queues.get(0);

			for (int i = 0; i < q.size(); i++) {
				TaskInterface<?> task = q.get(i);
				initApplicationType(task);
				
				if (task.getStatus() == DCWormsTags.READY) {

					Map<ResourceUnitName, ResourceUnit> choosenResources = chooseResourcesForExecution(resourceManager, task);
					if (choosenResources  != null) {
						addToSchedulingPlan(plan, task, choosenResources);
					} else {
						if(harnessIdleNodesToWork(task, resourceManager.getNodes()))
							i--;
					}
				}
			}
			turnOffIdleNodes(resourceManager.getNodes());
			break;
		}
		return plan;
	}
	
	private Map<ResourceUnitName, ResourceUnit> chooseResourcesForExecution(ClusterResourceManager resourceManager, TaskInterface<?> task) {

		Map<ResourceUnitName, ResourceUnit> map = new HashMap<ResourceUnitName, ResourceUnit>();
		
		List<Node> nodes = resourceManager.getNodes();
		List<Node> avNodes = filterNodes(nodes, task);
		if(avNodes.size() == 0)
			return null;

		for(Node node: avNodes){
			int cpuRequest;
			try {
				cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
			} catch (NoSuchFieldException e) {
				cpuRequest = 0;
			}
	
			if (cpuRequest != 0) {
	
				List<Core> cores = node.getProcessors().get(0).getCores();
				List<ComputingResource> choosenResources = new ArrayList<ComputingResource>();				
				for (int i = 0; i < cores.size() && cpuRequest > 0; i++) {
					if (cores.get(i).getStatus() == ResourceStatus.FREE) {
						choosenResources.add(cores.get(i));
						cpuRequest--;
					}
				}
				if (cpuRequest > 0) {
					continue;
				}
				ProcessingElements pe = new ProcessingElements();
				pe.addAll(choosenResources);
				map.put(StandardResourceUnitName.PE, pe);
				return map;
			}
		}
		return null;
	}

	private void turnOffIdleNodes(List<Node> nodes){
		for(Node node : nodes){
			Processor proc = node.getProcessors().get(0);
			int freeCores = 0;
			for(Core core: proc.getCores()){
				if(core.getStatus() == ResourceStatus.FREE)
					freeCores++;
			}
			
			if(freeCores == proc.getCores().size())
				node.getPowerInterface().setPowerState(StandardPowerStateName.OFF);
	
		}
	}
	
	private boolean harnessIdleNodesToWork(TaskInterface<?> task, List<Node> nodes){

		int cpuRequest;
		try {
			cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
		} catch (NoSuchFieldException e) {
			cpuRequest = 0;
		}
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if(node.getPowerInterface().getPowerState() == StandardPowerStateName.OFF){

				List<Core> cores = node.getProcessors().get(0).getCores();
				if (cores.size() < cpuRequest) {
					continue;
				}
				node.getPowerInterface().setPowerState(StandardPowerStateName.ON);
				return true;
			}
		}
		return false;
	}

	
	private List<Node> filterNodes(List<Node> nodes, TaskInterface<?> task){
		List<Node> filteredNodes = new ArrayList<Node>();
		for (Node node : nodes) {
			int cpuRequest;
			try {
				cpuRequest = Double.valueOf(task.getCpuCntRequest()).intValue();
			} catch (NoSuchFieldException e) {
				cpuRequest = 0;
			}

			if (cpuRequest != 0) {

				List<Core> cores = node.getProcessors().get(0).getCores();
				if (cores.size() < cpuRequest) {
					continue;
				}

				int freeCores = 0;
				for(Core core: cores){
					if(core.getStatus() == ResourceStatus.FREE)
						freeCores++;
				}
				
				if(freeCores != cores.size())
					continue;
				
				filteredNodes.add(node);
			}
		}
		
		return filteredNodes;
	}
	
}